static struct
{
  char *keysymstr;
  RTFCommand cmd;
}EditorKeyStrings[]=
{
  {"Up",RTFUp},
  {"Down",RTFDown},
  {"Left",RTFLeft},
  {"Right",RTFRight},
  {"PageUp",RTFPageUp},
  {"PageDown",RTFPageDown},
  {"Home",RTFHome},
  {"End",RTFEnd},
  {"StartOfText",RTFStartOfText},
  {"EndOfText",RTFEndOfText},
  {"PreviousWord",RTFPreviousWord},
  {"NextWord",RTFNextWord},
  {"DeleteCurrentChar",RTFDelete},
  {"DeletePreviousChar",RTFBackSpace},
  {"DeletePreviousWord",RTFDeletePreviousWord},
  {"DeleteNextWord",RTFDeleteNextWord},
  {"NewLine",RTFNewLine},

  {"Space",RTFSpace},
  {"Bold",RtfBold},
  {"Italic",RtfItalic},
  {"Fixed",RtfFixed},
  {"Underline",RtfUnderLine},
  {"IncreaseSize",RtfBigger},
  {"DecreaseSize",RtfBiggerOFF},
  {"Center",RtfCenter},
  {"FlushLeft",RtfFlushLeft},
  {"FlushRight",RtfFlushRight},
  {"IncreaseIndent",RtfIndent},
  {"DecreaseIndent",RtfIndentOFF},
  {"IncreaseIndentRight",RtfIndentRight},
  {"DecreaseIndentRight",RtfIndentRightOFF},
  {"SelectColor",RtfX_Color},
  {"Search",RTFSearch},
  {"RepeatSearchForward",RTFRepeatSearchForward},
  {"RepeatSearchBackward",RTFRepeatSearchBackward},
  {"RepeatLastSearch",RTFRepeatLastSearch},
  {"DeleteBlock",RTFRemoveBlock},
  {"DeleteToEOL",RTFDeleteToEOL}
};


void umtGRTFEditorImpl::LoadKeyMap(umtConfiguration* config)
{
  umtConfiguration* cur;
  char   cur_key[1024];
  char   cur_cmd[1024];
  int    i;
  
  DoneKeyMap();
  if(config==nil)return;

  for( cur=config->GetFirst() ; cur ; cur=config->GetNext(cur) )
    {
      cur_key[0]=0;
      cur_cmd[0]=0;
      if(!((cur->GetName()).IsEmpty()))
	strcpy(cur_key,cur->GetName());
      if(!((cur->GetParByNumber(0)).IsEmpty()))
	strcpy(cur_cmd,cur->GetParByNumber(0));

      if((cur_cmd[0]!=0) && (cur_key[0]!=0))
	{
	  for(
	      i=0 ;
	      i<sizeof(EditorKeyStrings)/sizeof(EditorKeyStrings[0]) ;
	      i++
	      )
	    {
	      if(!strcasecmp(cur_cmd,EditorKeyStrings[i].keysymstr))
		{
		  //	  fprintf(stderr,"%s : >%s<",cur_key,cur_cmd);
		  KeyMapStrings.append(new char[strlen(cur_key)+5]);
		  strcpy(KeyMapStrings.item(KeyMapStrings.count()-1),cur_key);
		  KeyMapCommands.append(EditorKeyStrings[i].cmd);
		  KeyMapChars.append('\0');
		  //		  fprintf(stderr," (>%s<)\n",KeyMapStrings.item(KeyMapStrings.count()-1));
		}
	    }
	  if(i==sizeof(EditorKeyStrings)/sizeof(EditorKeyStrings[0]))
	    {
	      KeyMapStrings.append(new char[strlen(cur_key)+5]);
	      strcpy(KeyMapStrings.item(KeyMapStrings.count()-1),cur_key);
	      KeyMapCommands.append(RtfUNKNOWN);
	      KeyMapChars.append(cur_cmd[0]);
	    }
	}
    }  
}


RTFCommand    umtGRTFEditorImpl::FindKey(const Event& e,char& c)
{
  char *str;
  char tmp[1024];

  c=0;
  
  str=XKeysymToString(e.keysym());
  if(str==nil)return RtfUNKNOWN;

  tmp[0]=0;
  if(e.control_is_down())strcat(tmp,"Ctrl+");
  if(e.meta_is_down())strcat(tmp,"Alt+");
  if(e.shift_is_down())strcat(tmp,"Shift+");
  
  switch(strlen(str))
    {
    case 0:return RtfUNKNOWN;
    case 1:
      if(
	 e.control_is_down() ||
	 e.meta_is_down()
	 )
	strcat(tmp,str);
      else
	{
	  char t[2]={0,0};
	  int  is_change=0;
	  
	  t[0]=str[0];
	  if(e.shift_is_down())is_change=1-is_change;
	  if(
	     e.capslock_is_down() &&
	     ((t[0]>='a' && t[0]<='z') || (t[0]>='A' && t[0]<='Z'))
	     )is_change=1-is_change;
	  
	  if(is_change)
	    {
	      static char down[]="`1234567890-=\\";
	      static char up[]  ="~!@#$%^&*()_+|";
	      int i;

	      if(t[0]>='a' && t[0]<='z')
		t[0]=toupper(t[0]);
	      else
		if(t[0]>='A' && t[0]<='Z')
		  t[0]=tolower(t[0]);
		else
		  for( i=0 ; i<sizeof(down) ; i++ )
		    if(t[0]==down[i])
		      {
			t[0]=up[i];
			break;
		      }
	    }
	  strcpy(tmp,t);
	}
      break;
    default:
      strcat(tmp,str);
      break;
    }

  //    fprintf(stderr," New string:>%s< \n",tmp);
  
  switch(strlen(tmp))
    {
    case 0:return RtfUNKNOWN;
    case 1:c=tmp[0];return RtfUNKNOWN;
    default:
      {
      	int i;
	for( i=0 ; i<KeyMapStrings.count() ; i++ )
	  if(!strcasecmp(KeyMapStrings.item(i),tmp))
	    {
	      if(KeyMapCommands.item(i)==RtfUNKNOWN)
		c=KeyMapChars.item(i);
	      return KeyMapCommands.item(i);
	    }

	for( i=0 ; i<Keyboards.count() ; i++ )
	    {
	      if(Keyboards.item(i)->HotKey()==nil)continue;
	      if(!strcasecmp(Keyboards.item(i)->HotKey(),tmp))
		{
		  c=(char)i;
		  return RTFSelectKeyboard;
		}		
	    }
      }
      return RtfUNKNOWN;
    }

}


void umtGRTFEditorImpl::DoneKeyMap(void)
{
  int i;

  for( i=0 ; i<KeyMapStrings.count() ; i++ )
    delete KeyMapStrings.item(i);
  KeyMapStrings.remove_all();
  KeyMapCommands.remove_all();
  KeyMapChars.remove_all();
}



